//  //
//  ###  ##### #   #               -= Confix Library =-                  //
// #   # #     ## ## ConfigVariable.h - Configuration file variable      //
// #     ####   ###                                                      //
// #     #      ###  Representation of a configuration file variable     //
// #   # #     ## ##                                                     //
//  ###  #     #   # R1             (C)2002 Markus Ewald -> License.txt  //
//  //
#ifndef CFX_CONFIGVARIABLE_H
#define CFX_CONFIGVARIABLE_H

#include "Confix/Config.h"

#include <string>

#ifdef CONFIX_NAMESPACE
  namespace CONFIX_NAMESPACE {
#endif CONFIX_NAMESPACE

//  //
//  CFX::CConfigVariable                                                 //
//  //
/// A configuration file variable
/** Stores a variable from a configuration file. It is designed in a way
    that the variable 'is a' value to be represented in the configuration
    file (thus the overloaded conversion and assignment operators) while
    it 'has a' name (.getName() / .setName()).

    Blank lines will be stored as empty values (T_NONE) which don't have
    a name. This includes comments, which have no valuable meaning for
    the parser (any line containing beginning with '#', '//' or ';').
    All other types of variables simply store their name and the value as
    well as their native type in the configuration file.

    Note that booleans are not directly supported in the configuration
    file. These are implicitely converted from other types (any number
    unequal to zero and any string with a length > 0). When you save a
    boolean, it will be written as either '1' or '0'.
    Any other way of handling booleans would open the problem of 
    localization and the decision of which identifiers to recognize 
    ('true'/'false', 'on'/'off', 'yes'/'no', ...). If an application
    requires this, it should query the values in question as strings and
    check these for matches with the desired names.
*/
class CONFIX_API CConfigVariable {
  public:
    /// Configuration variable types
    enum TYPE {
      T_NONE = 0,                                     ///< No value / empty
      T_COMMENT,                                      ///< Comment
      T_BOOL,                                         ///< Boolean value
      T_LONG,                                         ///< Long value
      T_HEX,                                          ///< Hex value
      T_DOUBLE,                                       ///< Double value
      T_STRING                                        ///< String value
    };

    /// Constructor
    CConfigVariable(void);
    /// Constructor
    CConfigVariable(const std::string &sContents);
    /// Copy constructor
    CConfigVariable(const CConfigVariable &Variable);
    /// Destructor
    ~CConfigVariable(void);

  //
  // CConfigVariable implementation
  //
  public:
    /// Get variable name
    const std::string &getName(void) const;
    /// Set variable name
    void setName(const std::string &sName);

    /// Get variable type
    TYPE getType(void) const;

    /// Retrieve as boolean
    bool getBool(void) const;
    /// Retrieve as long
    long getLong(void) const;
    /// Retrieve as double
    double getDouble(void) const;
    /// Retrieve as string
    std::string getString(void) const;

    /// Set to boolean
    void setBool(bool bValue);
    /// Set to long
    void setLong(long nValue);
    /// Set to double
    void setDouble(double dValue);
    /// Set to string
    void setString(const std::string &sValue);

    /// Assign copy
    CConfigVariable &operator =(const CConfigVariable &Value);
    /// Assign boolean
    CConfigVariable &operator =(bool bValue) { setBool(bValue); return *this; }
    /// Assign integer (for convenience)
    CConfigVariable &operator =(int nValue) { setLong(nValue); return *this; }
    /// Assign long
    CConfigVariable &operator =(long nValue) { setLong(nValue); return *this; }
    /// Assign double
    CConfigVariable &operator =(double dValue) { setDouble(dValue); return *this; }
    /// Assign char array (for convenience)
    CConfigVariable &operator =(const char *pszValue) { setString(pszValue); return *this; }
    /// Assign string
    CConfigVariable &operator =(const std::string &sValue) { setString(sValue); return *this; }

    /// Convert to boolean
    inline operator bool(void) const { return getBool(); }
    /// Convert to long
    inline operator long(void) const { return getLong(); }
    /// Convert to double
    operator double(void) const { return getDouble(); }
    /// Convert to string
    operator std::string(void) const { return getString(); }

    /// Clear variable
    void clearContents(void);
    /// Get configuration contents
    const std::string &getContents(void) const;
    /// Set configuration contents
    std::string::size_type setContents(const std::string &sContents, std::string::size_type Start = 0);

  private:
    /// Clear the value
    void clearValue(void);
    /// Scan for the assigned value
    std::string::size_type scanValue(const std::string &sString, std::string::size_type Start);
    /// Scan a comment
    std::string::size_type scanComment(const std::string &sString, std::string::size_type Start);
    /// Scan a string assignment
    std::string::size_type scanString(const std::string &sString, std::string::size_type Start);
    /// Scan a number assignment
    std::string::size_type scanNumber(const std::string &sString, std::string::size_type Start);

    std::string *m_psName;                            ///< Variable name
    TYPE         m_eType;                             ///< Variable type
    union {
      bool         m_bValue;                          ///< Boolean value
      long         m_lValue;                          ///< Long/Hex value
      double       m_dValue;                          ///< Double value
      std::string *m_psValue;                         ///< String value
    };
    
    mutable bool                    m_bChanged;       ///< Has changed
    mutable std::string            *m_psContents;     ///< File contents
    mutable std::string::size_type  m_NameStart;      ///< Name starting offset
    mutable std::string::size_type  m_NameLength;     ///< Name length
    mutable std::string::size_type  m_ValueStart;     ///< Value starting offset
    mutable std::string::size_type  m_ValueLength;    ///< Value length
};

#ifdef CONFIX_NAMESPACE
  }; // CONFIX_NAMESPACE
#endif CONFIX_NAMESPACE

#endif CFX_CONFIGVARIABLE_H
